@@ -1,6 +1,6 @@
 /*
     Rhpc : R HPC environment
-    Copyright (C) 2012-2015  Junji NAKANO and Ei-ji Nakama
+    Copyright (C) 2012-2018  Junji NAKANO and Ei-ji Nakama
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as published by
@@ -35,7 +35,7 @@ __inline static void Rhpc_mpi_lapply_seq_exit(int procs, MPI_Comm comm)
   MPI_Request *request=Calloc((procs-1),MPI_Request);
   MPI_Status  *status =Calloc((procs-1),MPI_Status);
 
-  SET_CMD(cmde, CMD_NAME_LAPPLY_SEQ , SUBCMD_EXIT, 0, 0 );
+  SET_CMD(cmde, CMD_NAME_LAPPLY_SEQ , SUBCMD_EXIT, 0, 0, 0 );
   for (i=1; i < procs ; i++){
     DPRINT("worker exit rank=%ld\n",i);
     _M(MPI_Isend(cmde,   (int)CMDLINESZ, MPI_INT, i, RHPC_CTRL_TAG, comm, &request[i-1]));
@@ -50,6 +50,7 @@ __inline static void Rhpc_mpi_lapply_seq_exit(int procs, MPI_Comm comm)
 __inline static void Rhpc_mpi_lapply_seq_send(R_xlen_t *workers,
 					      int procs,
 					      SEXP X,
+					      SEXP usequote,
 					      MPI_Comm comm)
 {
   R_xlen_t i,j;
@@ -77,7 +78,7 @@ __inline static void Rhpc_mpi_lapply_seq_send(R_xlen_t *workers,
   
   for (i=1; i < procs ; i++){
     workers[i]=xlength(VECTOR_ELT(XL,i-1));
-    REPROTECT(Xsel= Rhpc_serialize(VECTOR_ELT(XL,i-1)), ix2);
+    REPROTECT(Xsel= Rhpc_serialize_norealloc(VECTOR_ELT(XL,i-1)), ix2);
     SET_VECTOR_ELT(sendlist,i-1, Xsel);
   }
   
@@ -100,7 +101,7 @@ __inline static void Rhpc_mpi_lapply_seq_send(R_xlen_t *workers,
       R_xlen_t lens = xlength(VECTOR_ELT(sendlist,i-1));
       R_xlen_t cnts = lens / RHPC_SPLIT_SIZE;
       R_xlen_t mods = lens % RHPC_SPLIT_SIZE;
-      SET_CMD(cmds, CMD_NAME_LAPPLY_SEQ , SUBCMD_NORMAL, cnts, mods );
+      SET_CMD(cmds, CMD_NAME_LAPPLY_SEQ , SUBCMD_NORMAL, cnts, mods, INTEGER(usequote)[0] );
       _M(MPI_Isend(cmds,   (int)CMDLINESZ, MPI_INT, i, RHPC_CTRL_TAG, comm, &request[sendcalls]));
       sendcalls++;
       /*
@@ -129,7 +130,7 @@ __inline static void Rhpc_mpi_lapply_seq_send(R_xlen_t *workers,
 
 
 
-SEXP Rhpc_mpi_lapply_seq(SEXP cl, SEXP X, SEXP args)
+SEXP Rhpc_mpi_lapply_seq(SEXP cl, SEXP X, SEXP args, SEXP usequote)
 {
   R_xlen_t i;
 
@@ -141,6 +142,7 @@ SEXP Rhpc_mpi_lapply_seq(SEXP cl, SEXP X, SEXP args)
   R_xlen_t szi;
   R_xlen_t cnti;
   R_xlen_t modi;
+  int      dummy_usequote;
   int cmd[CMDLINESZ];
 
   R_xlen_t xlen = xlength(X); 
@@ -183,7 +185,7 @@ SEXP Rhpc_mpi_lapply_seq(SEXP cl, SEXP X, SEXP args)
   /* serialize */
   PROTECT(names);
   PROTECT(l_out);
-  PROTECT(out=Rhpc_serialize(args));
+  PROTECT(out=Rhpc_serialize_norealloc(args));
 
 
   /* cmd send */
@@ -191,7 +193,7 @@ SEXP Rhpc_mpi_lapply_seq(SEXP cl, SEXP X, SEXP args)
   cnti = szi/RHPC_SPLIT_SIZE;
   modi = szi%RHPC_SPLIT_SIZE;
   SET_CMD(cmd, CMD_NAME_LAPPLY_SEQ,
-	  SUBCMD_NORMAL,cnti, modi);
+	  SUBCMD_NORMAL,cnti, modi, INTEGER(usequote)[0]);
   _M(MPI_Bcast(cmd, CMDLINESZ, MPI_INT, 0, comm));
 
 
@@ -220,7 +222,7 @@ SEXP Rhpc_mpi_lapply_seq(SEXP cl, SEXP X, SEXP args)
 
   if(!isNull(names)) setAttrib(outlist, R_NamesSymbol, names);
   
-  Rhpc_mpi_lapply_seq_send(workers, procs, X, comm);
+  Rhpc_mpi_lapply_seq_send(workers, procs, X, usequote, comm);
 
   while( Rhpc_mpi_lapply_seq_worker_calls(workers,procs)){
 
@@ -241,7 +243,7 @@ SEXP Rhpc_mpi_lapply_seq(SEXP cl, SEXP X, SEXP args)
 	int msgcnt;
 	int calls;
 	_M(MPI_Recv(cmdr, CMDLINESZ, MPI_INT, wkr, RHPC_CTRL_TAG, comm, &stat));
-	GET_CMD(cmdr, &cmdmainr, &cmdsubr, &cntr, &modr);
+	GET_CMD(cmdr, &cmdmainr, &cmdsubr, &cntr, &modr, &dummy_usequote);
 	lenr = RHPC_SPLIT_SIZE * cntr + modr;
 	REPROTECT(indata=allocVector(RAWSXP,lenr),indata_ix);
 	
